const { EmbedBuilder } = require('discord.js');
const { error } = require('@helpers/Logger')
const HttpUtil = require('@helpers/HttpUtils');
const Utils = require('@helpers/Utils');
const { byteUnit } = require('@root/src/helpers/Units');

module.exports = {
  name: "nodes",
  description: "Lists Audio nodes",
  category: "UTILITY",
  command: {
    enabled: true,
  },
  slashCommand: {
    enabled: true
  },

  /**
   * @param {import('discord.js').Message} message
   */
  async messageRun(message) {
    await message.safeReply({
      embeds: [await fetchNodesAndTurnItToTable(message.client.config.MUSIC.LAVALINK_NODES)]
    })
  },

  /**
   * @param {import('discord.js').ChatInputCommandInteraction} interaction
   */
  async interactionRun(interaction) {
    await interaction.followUp({
      embeds: [await fetchNodesAndTurnItToTable(interaction.client.config.MUSIC.LAVALINK_NODES)]
    })
  },
};

async function fetchNodesAndTurnItToTable(nodes) {
  const fields = await Promise.all(nodes.map(async node => {
    let stat
    try {
      const res = await fetchNodeStat(node).then(HttpUtil.assertStatusOk)
      stat = await res.json()
    } catch(e) {
      error(node.id, e)
    }

    if (stat) {
      const { uptime, memory: { free, reservable }, cpu: { lavalinkLoad }, playingPlayers, players } = stat
      return {
        name: `:green_circle: ${node.id}`,
        value: [
          `❯ **Uptime**: ${Utils.formatTime(uptime)}`,
          `❯ **Memory**: ${byteUnit(free)} (${(free / reservable * 100).toPrecision(3)}%) of ${byteUnit(reservable)}`,
          `❯ **CPU**: ${(lavalinkLoad * 100).toFixed(1)}%`,
          `❯ **Players**: ${playingPlayers} / ${players}`,
        ].join('\n')
      }
    } else {
      return {
        name: `:red_circle: ${node.id}`,
        value: 'Node is offline'
      }
    }
  }))

  return new EmbedBuilder({
    title: 'Audio Nodes',
    color: 0x0088ff,
    fields: fields
  })
}

async function fetchNodeStat(node, connTimeout = 5000) {
  const baseUrl = (node.port !== 80 && node.secure) 
    ? `https://${node.host}:${node.port}` 
    : `http://${node.host}:${node.port}`;

  return HttpUtil.fetchWithConnTimeout(
    `${baseUrl}/v4/stats`,
    { headers: { Authorization: node.authorization } },
    connTimeout
  );
}
