const { trunc, log10, log, pow, ceil, min } = Math;

/**
 * Converts value to unit-based
 * @param {number} value Value
 * @param {ArrayLike<string>} names Unit names
 * @param base Base value (default: `1000`)
 * @param precision Number of precision (default: `3`)
 */
function valueUnit(value, names, base = 1000, precision = ceil(log10(base))) {
	if (!value) return '0'

	const ix = min(names.length - 1, trunc(log(value) / log(base)))
	if (ix === 0 && Number.isInteger(value)) return value + names[0]

	const iv = value / pow(base, ix)
	return (log10(iv) >= precision ? iv.toFixed(0) : iv.toPrecision(precision)) + names[ix]
}

/**
 * Create vault unit converter
 * @param {ArrayLike<string>} names Unit names
 * @param base Base value (default: `1000`)
 * @param precision Number of precision (default: `3`)
 * @returns { (value: number) => string }
 */
function createValueUnit(names,base = 1000,precision = ceil(log10(base))) {
	return (value) => valueUnit(value, names, base, precision)
}

const byteUnits = ["B", "KB", "MB", "GB"];
const byteUnit = createValueUnit(byteUnits, 1024, 3)

const abbreviations = ["", "k", "M", "B", 'T'];
const abbreviate = createValueUnit(abbreviations)

exports.default = valueUnit
exports.createValueUnit = createValueUnit
exports.byteUnits = byteUnits
exports.byteUnit = byteUnit
exports.abbreviations = abbreviations
exports.abbreviate = abbreviate